@@ -1,7 +1,9 @@
 import sys
+from functools import partial
 from typing import Callable, List, Optional, Sequence, Tuple, cast
 from typing_extensions import Literal
 from typing import Type as typing_Type
+from mypy import message_registry
 from mypy.plugin import Plugin, FunctionContext, MethodContext, CheckerPluginInterface
 from mypy.nodes import (
     ARG_POS,
@@ -30,17 +32,13 @@ from mypy.types import (
 )
 from mypy.typeops import make_simplified_union
 from mypy.checker import TypeChecker
+from packaging.version import parse as parse_version
 
 
 class TrioPlugin(Plugin):
     def get_function_hook(
         self, fullname: str
     ) -> Optional[Callable[[FunctionContext], Type]]:
-        if fullname in (
-            "contextlib.asynccontextmanager",
-            "async_generator.asynccontextmanager",
-        ):
-            return args_invariant_decorator_callback
         if fullname == "trio_typing.takes_callable_and_args":
             return takes_callable_and_args_callback
         if fullname == "async_generator.async_generator":
@@ -51,32 +49,15 @@ class TrioPlugin(Plugin):
             return yield_from_callback
         return None
 
-    def get_method_hook(
+
+class TrioPlugin13(TrioPlugin):
+    def get_function_hook(
         self, fullname: str
-    ) -> Optional[Callable[[MethodContext], Type]]:
-        if fullname == "trio_typing.TaskStatus.started":
-            return started_callback
-        return None
+    ) -> Optional[Callable[[FunctionContext], Type]]:
+        if fullname == "trio_typing.takes_callable_and_args":
+            return partial(takes_callable_and_args_callback, has_type_var_default=False)
 
-
-def args_invariant_decorator_callback(ctx: FunctionContext) -> Type:
-    """Infer a better return type for @asynccontextmanager,
-    @async_generator, and other decorators that affect the return
-    type but not the argument types of the function they decorate.
-    """
-    # (adapted from the @contextmanager support in mypy's builtin plugin)
-    if ctx.arg_types and len(ctx.arg_types[0]) == 1:
-        arg_type = get_proper_type(ctx.arg_types[0][0])
-        ret_type = get_proper_type(ctx.default_return_type)
-        if isinstance(arg_type, CallableType) and isinstance(ret_type, CallableType):
-            return ret_type.copy_modified(
-                arg_types=arg_type.arg_types,
-                arg_kinds=arg_type.arg_kinds,
-                arg_names=arg_type.arg_names,
-                variables=arg_type.variables,
-                is_ellipsis_args=arg_type.is_ellipsis_args,
-            )
-    return ctx.default_return_type
+        return super().get_function_hook(fullname)
 
 
 def decode_agen_types_from_return_type(
@@ -183,25 +164,23 @@ def async_generator_callback(ctx: FunctionContext) -> Type:
     YieldType[bool], SendType[int]]`` without the plugin.
     """
 
-    # Apply the common logic to not change the arguments of the
-    # decorated function
-    new_return_type = args_invariant_decorator_callback(ctx)
-    if not isinstance(new_return_type, CallableType):
-        return new_return_type
-    agen_return_type = get_proper_type(new_return_type.ret_type)
+    decorator_return_type = ctx.default_return_type
+    if not isinstance(decorator_return_type, CallableType):
+        return decorator_return_type
+    agen_return_type = get_proper_type(decorator_return_type.ret_type)
     if (
         isinstance(agen_return_type, Instance)
         and agen_return_type.type.fullname == "trio_typing.CompatAsyncGenerator"
         and len(agen_return_type.args) == 3
     ):
-        return new_return_type.copy_modified(
+        return decorator_return_type.copy_modified(
             ret_type=agen_return_type.copy_modified(
                 args=list(
                     decode_agen_types_from_return_type(ctx, agen_return_type.args[2])
                 )
             )
         )
-    return new_return_type
+    return decorator_return_type
 
 
 def decode_enclosing_agen_types(ctx: FunctionContext) -> Tuple[Type, Type]:
@@ -264,6 +243,7 @@ def yield_callback(ctx: FunctionContext) -> Type:
             subtype=arg_type,
             supertype=yield_type,
             context=ctx.context,
+            msg=message_registry.INCOMPATIBLE_TYPES,
             subtype_label="yield_ argument",
             supertype_label="declared YieldType",
         )
@@ -299,6 +279,7 @@ def yield_from_callback(ctx: FunctionContext) -> Type:
             subtype=their_yield_type,
             supertype=our_yield_type,
             context=ctx.context,
+            msg=message_registry.INCOMPATIBLE_TYPES,
             subtype_label="yield_from_ argument YieldType",
             supertype_label="local declared YieldType",
         )
@@ -306,6 +287,7 @@ def yield_from_callback(ctx: FunctionContext) -> Type:
             subtype=our_send_type,
             supertype=their_send_type,
             context=ctx.context,
+            msg=message_registry.INCOMPATIBLE_TYPES,
             subtype_label="local declared SendType",
             supertype_label="yield_from_ argument SendType",
         )
@@ -316,6 +298,7 @@ def yield_from_callback(ctx: FunctionContext) -> Type:
                 "typing.AsyncIterable", [our_yield_type]
             ),
             context=ctx.context,
+            msg=message_registry.INCOMPATIBLE_TYPES,
             subtype_label="yield_from_ argument type",
             supertype_label="expected iterable type",
         )
@@ -323,26 +306,9 @@ def yield_from_callback(ctx: FunctionContext) -> Type:
     return ctx.default_return_type
 
 
-def started_callback(ctx: MethodContext) -> Type:
-    """Raise an error if task_status.started() is called without an argument
-    and the TaskStatus is not declared to accept a result of type None.
-    """
-    self_type = get_proper_type(ctx.type)
-    if (
-        (not ctx.arg_types or not ctx.arg_types[0])
-        and isinstance(self_type, Instance)
-        and self_type.args
-        and not isinstance(get_proper_type(self_type.args[0]), NoneTyp)
-    ):
-        ctx.api.fail(
-            "TaskStatus.started() requires an argument for types other than "
-            "TaskStatus[None]",
-            ctx.context,
-        )
-    return ctx.default_return_type
-
-
-def takes_callable_and_args_callback(ctx: FunctionContext) -> Type:
+def takes_callable_and_args_callback(
+    ctx: FunctionContext, has_type_var_default: bool = True
+) -> Type:
     """Automate the boilerplate for writing functions that accept
     arbitrary positional arguments of the same type accepted by
     a callable.
@@ -492,17 +458,33 @@ def takes_callable_and_args_callback(ctx: FunctionContext) -> Type:
                     ),
                 )
             )
-            type_var_types.append(
-                TypeVarType(
-                    "__T{}".format(arg_idx),
-                    "__T{}".format(arg_idx),
-                    -len(fn_type.variables) - arg_idx - 1,
-                    [],
-                    ctx.api.named_generic_type("builtins.object", []),
-                    line=ctx.context.line,
-                    column=ctx.context.column,
+
+            if has_type_var_default:
+                type_var_types.append(
+                    TypeVarType(
+                        "__T{}".format(arg_idx),
+                        "__T{}".format(arg_idx),
+                        -len(fn_type.variables) - arg_idx - 1,
+                        [],
+                        ctx.api.named_generic_type("builtins.object", []),
+                        line=ctx.context.line,
+                        column=ctx.context.column,
+                        default=AnyType(TypeOfAny.from_omitted_generics),
+                    )
                 )
-            )
+            else:
+                type_var_types.append(
+                    TypeVarType(
+                        "__T{}".format(arg_idx),
+                        "__T{}".format(arg_idx),
+                        -len(fn_type.variables) - arg_idx - 1,
+                        [],
+                        ctx.api.named_generic_type("builtins.object", []),
+                        line=ctx.context.line,
+                        column=ctx.context.column,
+                    )  # type: ignore[call-arg]
+                )
+
         return Overloaded(expanded_fns)
 
     except ValueError as ex:
@@ -513,4 +495,11 @@ def takes_callable_and_args_callback(ctx: FunctionContext) -> Type:
 
 
 def plugin(version: str) -> typing_Type[Plugin]:
-    return TrioPlugin
+    mypy_version = parse_version(version)
+
+    if mypy_version < parse_version("1.0"):
+        raise RuntimeError("This version of trio-typing requires at least mypy 1.0.")
+    elif mypy_version < parse_version("1.4"):
+        return TrioPlugin13
+    else:
+        return TrioPlugin
